!#************************************************************************
!# E.S.O. - 3.60 Upgrade Project
!#
!#
!# who		when		what
!# ----------	--------	------------------------
!# c.levin	920601		created
!# p.leisy	980806		VLT Software adaptation
!# p.leisy	980815		Focus consistancy check
!# kbanse	100108		implement verifications
!# kbanse	100407		last modif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
!#   NAME:	EXTFocusSeq
!# 
!#   SYNOPSIS:	EXTFocusSeq
!#   
!#   DESCRIPTION:	
!#   Computes the Focus Offset to be applied to M2
!#    
!#   ENVIRONMENT:	
!#   PCO must be running together with a background MIDAS session	
!#
!#   RETURN VALUES:
!#   Focus Offset is written in the keyword OUTPUTR(1)
! ------------------------------------------------------
!
! PARAMETERS:
!    P1: start-x (default = 0.)
!    P2: step-x  (default = 1.)
!    P3: Step Number
!
defi/par P1 0. N "Enter the X start [0] : "
defi/par P2 1. N "Enter the X step [1]: "
defi/par P3 ?  N "Enter the Step number : "

!
! initialization
defi/loc focx/d/1/1	0.0
defi/loc focy/d/1/1	0.0
defi/loc focSTART/r/1/2	{P1}
defi/loc focSTEP/r/1/2	{P2}
defi/loc numSTEP/d/1/2	{P3}
defi/loc statFoc/i/1/3	0,0,0

define/local focRANG/r/1/3  0.0 all +lower	!because of @@ kcompare...

defi/loc sigmax/d/1/1	0.0
defi/loc sigmay/d/1/1	0.0
defi/loc TelFocus/r/1/1	0.0
defi/loc aux/d/1/1	0.0
defi/loc fox/d/1/1	0.0
defi/loc sign/d/1/1	0.0

!-----------------------------------------------------
! setup the graphics window
set/grap default
set/grap pmode=1

!-----------------------------------------------------
! getting the coordinates...
!
$xset b 100 2500 200
write/out "Mark stars sequence selection."
write/out "Remember last star in the sequence is the one with double telescope offset ..."
write/out " "

clear/ch ov
cent/gauss {idimemc},thrufocus tfocus
if outputi(1) .eq. 0 then
   write/out
   write/out Invalid number of coordinates.
   errsum = errsum + 1
   return
endif
copy/tk tfocus :xcen,:ycen,:xerr,:yerr,:xfwhm,:yfwhm,:icent @5 -
outputr/r/1/7
write/keyw rcorr -
1465.7174,1772.0569,0.44937E-01,0.30249E-01,9.6964,9.6857,4122.0

@@ kcompare focu010:rcorr outputr 1,2 0.05 3,4 0.0001 5,6 0.001
@@ kcompare focu011:rcorr outputr 7,7 0.0001

!
set/format F9.4
focRANG(1) = {focSTART}
focRANG(2) = {focSTART} + {focSTEP}*({numSTEP} - 1 )

write/keyw rcorr -3.5,-2.7
@@ kcompare focu015:rcorr focRANG 1,2 0.05

! Original Tel Focus
focRANG(3) = {focSTART} +{focSTEP}*({numSTEP} -1 )/2
write/out " "
write/out " *************************************** "
write/out " "
write/out "Focus Interval checked : {focRANG(1)} - {focRANG(2)}"
write/out " "
write/out " *************************************** "
set/format
!
set/midas output=no	! we don't want output on the screen...

! fit-x computation
comp/tab tfocus :step = SEQ
comp/tab tfocus :Tel_Focus = {focSTART} + (:step - 1)*{focSTEP}
regr/poly tfocus :xfwhm :step 2
save/regr tfocus test
comp/regr tfocus :fitx = test(:step)

! compare column :fitx of tfocus.tbl with expected values
do inputi = 1 9
 outputr({inputi}) = m$value(tfocus.tbl,:fitx,@{inputi})
enddo
write/keyw rcorr -
13.66091,12.1873,11.03647,10.20843,9.703176,9.520704,9.661018,10.12412,10.91 
@@ kcompare focu020:rcorr outputr 1,9 0.0005

focx = {focSTART} + {focSTEP}*({outputd(2)}/(-2.*{outputd(3)})) - {focSTEP}
IF focx .GE. {focRANG(2)} THEN
write/out "focx false : {focx} {focRANG(2)}"
   statFoc(1) = 1
   errsum = errsum + 1
ENDIF
IF focx .LE. {focRANG(1)} THEN
write/out "focx false : {focx} {focRANG(1)}"
   statFoc(1) = 1
   errsum = errsum + 1
ENDIF
!
! fit-y computation
regr/poly tfocus :yfwhm :step 2
save/regr tfocus test
comp/regr tfocus :fity = test(:step)

! compare column :fity of tfocus.tbl with expected values
do inputi = 1 9
 outputr({inputi}) = m$value(tfocus.tbl,:fity,@{inputi})
enddo
write/keyw rcorr -
13.87974,12.45999,11.37492,10.6245,10.20876,10.12768,10.38127,10.96952,11.89244
@@ kcompare focu030:rcorr outputr 1,9 0.0005

set/midas output=yes
focy = {focSTART} + {focSTEP}*({outputd(2)}/(-2.*{outputd(3)})) - {focSTEP}
IF focy .GE. {focRANG(2)} THEN
write/out "focy false : {focy} {focRANG(2)}"
  statFoc(2) = 1
ENDIF
IF focy .LE. {focRANG(1)} THEN
write/out "focy false : {focy} {focRANG(1)}"
  statFoc(2) = 1
ENDIF
statFoc(3) = {statFoc(1)}+{statFoc(2)}

!
! compute the scaling for plots (big job)
comp/tab tfocus :maxim = MAX(:xfwhm,:yfwhm)
sort/tab tfocus :maxim(-)
defi/loc maxim/r/1/1 {tfocus,:maxim,1}
maxim = {maxim} + 0.2 	! let's give a margin value
comp/tab tfocus :minim = MIN(:xfwhm,:yfwhm)
sort/tab tfocus :minim
defi/loc minim/r/1/1 {tfocus,:minim,1}
minim = {minim} - 0.2 	! let's give a margin value
set/grap yaxis={minim},{maxim}

outputr(1) = minim
outputr(2) = maxim
write/keyw rcorr 8.59725,13.9302
@@ kcompare focu040:rcorr outputr 1,2 0.001
!
! plot (x,y)-value versus (x,y)-sigma
sort/tab tfocus :step
set/grap stype=6 colour=1 ! cross(x), black
plot/tab tfocus :Tel_Focus :xfwhm
set/grap stype=5 colour=4 ! cross(+), blue
over/tab tfocus :Tel_Focus :yfwhm

!
! plot of the fit
set/grap stype=0 ltype=1 ! no symbol, solid line
set/grap colour=1 ! black
over/tab tfocus :Tel_Focus :fitx
set/grap colour=4 ! blue
over/tab tfocus :Tel_Focus :fity 

!
! focus computation
!
IF statFoc(3) .EQ. 0 THEN
   aux  = {focx} * {focy}
ELSE
   IF statFoc(1) .EQ. 0 THEN
      aux  = {focx} * {focx}
      write/out " ********************************************* "
      write/out "  Focus not accurate ... False Y determination "
      write/out " ********************************************* "
      errsum = errsum + 1
   ENDIF
   IF statFoc(2) .EQ. 0 THEN
      aux  = {focy} * {focy}
      write/out " ********************************************* "
      write/out "  Focus not accurate ... False X determination "
      write/out " ********************************************* "
      errsum = errsum + 1
   ENDIF
ENDIF

! The 2 focus determinations are positive
! aux  = M$ABS({aux})
! fox  = M$ABS({focx})
! sign = {focx}/{fox}
! focus = {sign} * M$SQRT({aux})

TelFocus = M$SQRT({aux}) * {focx} / M$ABS({focx})
$xset b 100 2500 200
write/out "{TelFocus}"

set/midas output=no

! residual computation
comp/tab tfocus :residx2 = (:xfwhm - :fitx) ** 2
comp/tab tfocus :residy2 = (:yfwhm - :fity) ** 2
stat/tab tfocus :residx2
write/key rcorr 4.52811e-05,1.14351,0.417489,0.337272
@@ kcompare focu050:rcorr outputr 1,1 0.0000001 2,4 0.0001
sigmax = outputr(3)

statis/table tfocus :residy2
write/key rcorr 0.0486303,0.924389,0.521262,0.354206
@@ kcompare focu055:rcorr outputr 1,1 0.0000001 2,4 0.0001
sigmay = outputr(3)
!
set/midas output=yes
set/format ,f12.2			!for double keywords
!
write/out " "
write/out "X: BLACK    y: BLUE"
write/out " "
write/out "FOCUS:  X = {focx}  Y = {focy}   MEAN = {TelFocus}"
write/out  " "
set/format F12.3
write/out "sigma-x = {sigmax}   sigma-y = {sigmay}"
set/grap default

outputr(1) = focx
outputr(2) = focy
outputr(3) = TelFocus
outputr(4) = sigmax
outputr(5) = sigmay
write/keyw rcorr -2.99,-3.03,-3.01,0.417,0.521
@@ kcompare focu060:rcorr outputr 1,5 0.005
!
!
write/out " "
write/out " *************************************** "
write/out " "
IF statFoc(3) .NE. 2 THEN
  	write/out "Telescope Final Focus  = {TelFocus}"
ELSE
  	write/out "  No Focus determination ....
 	write/out " "
  	write/out " Move manually the Telescope Focus to a better value"
  	write/out " Try with more steps ....."
ENDIF
write/out " "
write/out " *************************************** "

!------------------------------------------------------------------------
! output for Templates
! 
outputR(1) = {TelFocus}

clear/ch ov


